'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initOption = initOption;
exports.createChart = createChart;
Highcharts.setOptions({
  lang: {
    numericSymbols: null // 不换算单位
  }
});

//初始化图表配置参数
function initOption(render) {
  var type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'line';
  var title = arguments[2];
  var subtitle = arguments[3];

  var option = {
    chart: {
      renderTo: render,
      zoomType: 'xy',
      type: type
    },
    title: {
      text: title,
      style: { "fontSize": "14px" }
    },
    subtitle: {
      text: subtitle,
      style: { "fontSize": "12px" }
    },
    colors: ["#48A7E7", "#943A9E", "#FEC633", "#4367B1", "#36A23F", "#F9494B", "#AA7EEB", "#FF7800", "#38DDF5", "#70CB3F", "#C45051", "#CCCCCC", "#30D0B1", "#F8BB00", "#41C4F3", "#05C705", "#FF63EE", "#97B7E8", "#80E5A1", "#EDEA0C", "#88D0FF", "#FF694E", "#DC99FF", "#66F2FF", "#FFB23F", "#63F1D6", "#FF9F8E"],
    xAxis: [{
      categories: null
    }],
    yAxis: [{ // Primary yAxis
      title: {
        text: null
      }
    }, { // Secondary yAxis
      title: {
        text: null
      },
      opposite: true
    }],
    tooltip: {
      crosshairs: true,
      shared: true,
      valueDecimals: 2
    },
    credits: {
      text: ''
    },
    legend: {
      borderColor: null
    },
    plotOptions: {},
    series: []
  };

  return option;
}

//创建图表
function createChart(chartData, option, seriesOption) {
  var autoRotation = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;

  if (chartData) {
    //设置x轴数据
    if (chartData.xAxisCategorieList && chartData.xAxisCategorieList.length > 0) {
      option.xAxis[0].categories = chartData.xAxisCategorieList;
      if (chartData.xAxisCategorieList.length > 10 && autoRotation) {
        if (option.xAxis[0].labels) {
          option.xAxis[0].labels.rotation = -45;
        } else {
          option.xAxis[0].labels = { rotation: -45 };
        }
      }
    }
    //设置y轴数据
    if (chartData.seriesDataList && chartData.seriesDataList.length > 0) {
      for (var i = 0; i < chartData.seriesDataList.length; i++) {
        if (seriesOption && seriesOption[i]) {
          for (var attr in seriesOption[i]) {
            chartData.seriesDataList[i][attr] = seriesOption[i][attr];
          }
        }
        option.series.push(chartData.seriesDataList[i]);
      }
    }
  }

  var chart = new Highcharts.Chart(option);
  if (!chart.hasData()) {
    chart.hideNoData();
    chart.showNoData("没有数据");
  }

  return chart;
}