'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = babelHelpers.interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = babelHelpers.interopRequireDefault(_propTypes);

var _baseStyles = require('./baseStyles');

var _baseStyles2 = babelHelpers.interopRequireDefault(_baseStyles);

var _BurgerIcon = require('./BurgerIcon');

var _BurgerIcon2 = babelHelpers.interopRequireDefault(_BurgerIcon);

var _CrossIcon = require('./CrossIcon');

var _CrossIcon2 = babelHelpers.interopRequireDefault(_CrossIcon);

exports.default = function (styles) {
  var Menu = function (_Component) {
    babelHelpers.inherits(Menu, _Component);

    function Menu(props) {
      babelHelpers.classCallCheck(this, Menu);

      var _this = babelHelpers.possibleConstructorReturn(this, _Component.call(this, props));

      _this.state = {
        isOpen: props && typeof props.isOpen !== 'undefined' ? props.isOpen : false
      };
      return _this;
    }

    Menu.prototype.toggleMenu = function toggleMenu() {
      var _this2 = this;

      var newState = { isOpen: !this.state.isOpen };

      this.applyWrapperStyles();

      this.setState(newState, function () {
        _this2.props.onStateChange(newState);

        // Timeout ensures wrappers are cleared after animation finishes.
        _this2.timeoutId && clearTimeout(_this2.timeoutId);
        _this2.timeoutId = setTimeout(function () {
          _this2.timeoutId = null;
          if (!newState.isOpen) {
            _this2.applyWrapperStyles(false);
          }
        }, 500);
      });
    };

    // Applies component-specific styles to external wrapper elements.


    Menu.prototype.applyWrapperStyles = function applyWrapperStyles() {
      var set = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

      if (this.props.bodyClassName) {
        var body = document.querySelector('body');
        body.classList[set ? 'add' : 'remove'](this.props.bodyClassName);
      }

      if (styles.pageWrap && this.props.pageWrapId) {
        this.handleExternalWrapper(this.props.pageWrapId, styles.pageWrap, set);
      }

      if (styles.outerContainer && this.props.outerContainerId) {
        this.handleExternalWrapper(this.props.outerContainerId, styles.outerContainer, set);
      }
    };

    // Sets or unsets styles on DOM elements outside the menu component.
    // This is necessary for correct page interaction with some of the menus.
    // Throws and returns if the required external elements don't exist,
    // which means any external page animations won't be applied.


    Menu.prototype.handleExternalWrapper = function handleExternalWrapper(id, wrapperStyles, set) {
      var html = document.querySelector('html');
      var body = document.querySelector('body');
      var wrapper = document.getElementById(id);

      if (!wrapper) {
        console.error("Element with ID '" + id + "' not found");
        return;
      }

      var builtStyles = this.getStyle(wrapperStyles);

      for (var prop in builtStyles) {
        if (builtStyles.hasOwnProperty(prop)) {
          wrapper.style[prop] = set ? builtStyles[prop] : '';
        }
      }

      // Prevent any horizontal scroll.
      [html, body].forEach(function (element) {
        element.style['overflow-x'] = set ? 'hidden' : '';
      });
    };

    // Builds styles incrementally for a given element.


    Menu.prototype.getStyles = function getStyles(el, index, inline) {
      var propName = 'bm' + el.replace(el.charAt(0), el.charAt(0).toUpperCase());

      // Set base styles.
      var output = _baseStyles2.default[el] ? this.getStyle(_baseStyles2.default[el]) : {};

      // Add animation-specific styles.
      if (styles[el]) {
        output = babelHelpers.extends({}, output, this.getStyle(styles[el], index + 1));
      }

      // Add custom styles.
      if (this.props.styles[propName]) {
        output = babelHelpers.extends({}, output, this.props.styles[propName]);
      }

      // Add element inline styles.
      if (inline) {
        output = babelHelpers.extends({}, output, inline);
      }

      return output;
    };

    Menu.prototype.getStyle = function getStyle(style, index) {
      var width = this.props.width;
      if (typeof width !== 'string') width = width + 'px';

      return style(this.state.isOpen, width, this.props.right, index);
    };

    Menu.prototype.listenForClose = function listenForClose(e) {
      e = e || window.event;

      if (this.state.isOpen && (e.key === 'Escape' || e.keyCode === 27)) {
        this.toggleMenu();
      }
    };

    Menu.prototype.componentWillMount = function componentWillMount() {
      if (!styles) {
        throw new Error('No styles supplied');
      }

      // Allow initial open state to be set by props.
      if (this.props.isOpen) {
        this.toggleMenu();
      }
    };

    Menu.prototype.componentDidMount = function componentDidMount() {
      /**
       * @yuhongda 2017-09-30
       */
      // window.onkeydown = this.listenForClose.bind(this);

      // Allow initial open state to be set by props for animations with wrapper elements.
      if (this.props.isOpen) {
        this.toggleMenu();
      }
    };

    Menu.prototype.componentWillUnmount = function componentWillUnmount() {
      window.onkeydown = null;

      this.applyWrapperStyles(false);
    };

    Menu.prototype.componentDidUpdate = function componentDidUpdate() {
      if (styles.svg) {
        var morphShape = _reactDom2.default.findDOMNode(this, 'bm-morph-shape');
        var path = styles.svg.lib(morphShape).select('path');

        if (this.state.isOpen) {
          // Animate SVG path.
          styles.svg.animate(path);
        } else {
          // Reset path (timeout ensures animation happens off screen).
          setTimeout(function () {
            path.attr('d', styles.svg.pathInitial);
          }, 300);
        }
      }
    };

    Menu.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
      if (typeof nextProps.isOpen !== 'undefined' && nextProps.isOpen !== this.state.isOpen) {
        this.toggleMenu();
      }
    };

    Menu.prototype.render = function render() {
      var _this3 = this;

      return _react2.default.createElement(
        'div',
        null,
        !this.props.noOverlay && _react2.default.createElement('div', {
          className: 'bm-overlay ' + this.props.overlayClassName,
          onClick: function onClick() {
            return !_this3.props.disableOverlayClick && _this3.toggleMenu();
          },
          style: this.getStyles('overlay')
        }),
        _react2.default.createElement(
          'div',
          {
            id: this.props.id,
            className: 'bm-menu-wrap ' + this.props.className,
            style: this.getStyles('menuWrap')
          },
          styles.svg && _react2.default.createElement(
            'div',
            { className: 'bm-morph-shape ' + this.props.morphShapeClassName, style: this.getStyles('morphShape') },
            _react2.default.createElement(
              'svg',
              { width: '100%', height: '100%', viewBox: '0 0 100 800', preserveAspectRatio: 'none' },
              _react2.default.createElement('path', { d: styles.svg.pathInitial })
            )
          ),
          _react2.default.createElement(
            'div',
            { className: 'bm-menu ' + this.props.menuClassName, style: this.getStyles('menu') },
            _react2.default.createElement(
              'nav',
              { className: 'bm-item-list ' + this.props.itemListClassName, style: this.getStyles('itemList') },
              _react2.default.Children.map(this.props.children, function (item, index) {
                if (item) {
                  var extraProps = {
                    key: index,
                    style: _this3.getStyles('item', index, item.props.style)
                  };
                  return _react2.default.cloneElement(item, extraProps);
                }
              })
            )
          ),
          this.props.customCrossIcon !== false && _react2.default.createElement(
            'div',
            { style: this.getStyles('closeButton') },
            _react2.default.createElement(_CrossIcon2.default, {
              onClick: function onClick() {
                return _this3.toggleMenu();
              },
              styles: this.props.styles,
              customIcon: this.props.customCrossIcon,
              className: this.props.crossButtonClassName,
              crossClassName: this.props.crossClassName
            })
          )
        ),
        this.props.customBurgerIcon !== false && _react2.default.createElement(
          'div',
          { style: this.getStyles('burgerIcon') },
          _react2.default.createElement(_BurgerIcon2.default, {
            onClick: function onClick() {
              return _this3.toggleMenu();
            },
            styles: this.props.styles,
            customIcon: this.props.customBurgerIcon,
            className: this.props.burgerButtonClassName,
            barClassName: this.props.burgerBarClassName
          })
        )
      );
    };

    return Menu;
  }(_react.Component);

  Menu.propTypes = {
    bodyClassName: _propTypes2.default.string,
    burgerBarClassName: _propTypes2.default.string,
    burgerButtonClassName: _propTypes2.default.string,
    crossButtonClassName: _propTypes2.default.string,
    crossClassName: _propTypes2.default.string,
    customBurgerIcon: _propTypes2.default.oneOfType([_propTypes2.default.element, _propTypes2.default.oneOf([false])]),
    customCrossIcon: _propTypes2.default.oneOfType([_propTypes2.default.element, _propTypes2.default.oneOf([false])]),
    disableOverlayClick: _propTypes2.default.bool,
    id: _propTypes2.default.string,
    isOpen: _propTypes2.default.bool,
    itemListClassName: _propTypes2.default.string,
    menuClassName: _propTypes2.default.string,
    morphShapeClassName: _propTypes2.default.string,
    noOverlay: _propTypes2.default.bool,
    onStateChange: _propTypes2.default.func,
    outerContainerId: styles && styles.outerContainer ? _propTypes2.default.string.isRequired : _propTypes2.default.string,
    overlayClassName: _propTypes2.default.string,
    pageWrapId: styles && styles.pageWrap ? _propTypes2.default.string.isRequired : _propTypes2.default.string,
    right: _propTypes2.default.bool,
    styles: _propTypes2.default.object,
    width: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string])
  };

  Menu.defaultProps = {
    bodyClassName: '',
    burgerBarClassName: '',
    burgerButtonClassName: '',
    className: '',
    crossButtonClassName: '',
    crossClassName: '',
    id: '',
    itemListClassName: '',
    menuClassName: '',
    morphShapeClassName: '',
    noOverlay: false,
    onStateChange: function onStateChange() {},
    outerContainerId: '',
    overlayClassName: '',
    pageWrapId: '',
    styles: {},
    width: 300
  };

  return Menu;
};