'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = babelHelpers.interopRequireDefault(_propTypes);

var _d = require('d3');

require('./style.css');

var Link = function (_React$PureComponent) {
  babelHelpers.inherits(Link, _React$PureComponent);

  function Link(props) {
    babelHelpers.classCallCheck(this, Link);

    var _this = babelHelpers.possibleConstructorReturn(this, _React$PureComponent.call(this, props));

    _this.state = {
      initialStyle: {
        opacity: 0
      }
    };
    return _this;
  }

  Link.prototype.componentDidMount = function componentDidMount() {
    this.applyOpacity(1);
  };

  Link.prototype.componentWillLeave = function componentWillLeave(done) {
    this.applyOpacity(0, done);
  };

  Link.prototype.applyOpacity = function applyOpacity(opacity) {
    var done = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};
    var transitionDuration = this.props.transitionDuration;


    (0, _d.select)(this.link).transition().duration(transitionDuration).style('opacity', opacity).each('end', done);
  };

  Link.prototype.diagonalPath = function diagonalPath(linkData, orientation) {
    var diagonal = _d.svg.diagonal().projection(function (d) {
      return orientation === 'horizontal' ? [d.y, d.x] : [d.x, d.y];
    });
    return diagonal(linkData);
  };

  Link.prototype.straightPath = function straightPath(linkData, orientation) {
    var straight = _d.svg.line().interpolate('basis').x(function (d) {
      return d.x;
    }).y(function (d) {
      return d.y;
    });

    var data = [{ x: linkData.source.x, y: linkData.source.y }, { x: linkData.target.x, y: linkData.target.y }];

    if (orientation === 'horizontal') {
      data = [{ x: linkData.source.y, y: linkData.source.x }, { x: linkData.target.y, y: linkData.target.x }];
    }

    return straight(data);
  };

  Link.prototype.elbowPath = function elbowPath(d, orientation) {
    return orientation === 'horizontal' ? 'M' + d.source.y + ',' + d.source.x + 'V' + d.target.x + 'H' + d.target.y : 'M' + d.source.x + ',' + d.source.y + 'V' + d.target.y + 'H' + d.target.x;
  };

  Link.prototype.drawPath = function drawPath() {
    var _props = this.props,
        linkData = _props.linkData,
        orientation = _props.orientation,
        pathFunc = _props.pathFunc;


    if (pathFunc === 'elbow') {
      return this.elbowPath(linkData, orientation);
    }

    if (pathFunc === 'straight') {
      return this.straightPath(linkData, orientation);
    }

    return this.diagonalPath(linkData, orientation);
  };

  Link.prototype.render = function render() {
    var _this2 = this;

    var styles = this.props.styles;

    return _react2.default.createElement('path', {
      key: this.props.key,
      ref: function ref(l) {
        _this2.link = l;
      },
      style: babelHelpers.extends({}, this.state.initialStyle, styles),
      className: 'linkBase',
      d: this.drawPath()
    });
  };

  return Link;
}(_react2.default.PureComponent);

exports.default = Link;


Link.defaultProps = {
  styles: {}
};

Link.propTypes = {
  linkData: _propTypes2.default.object.isRequired,
  orientation: _propTypes2.default.oneOf(['horizontal', 'vertical']).isRequired,
  pathFunc: _propTypes2.default.oneOf(['diagonal', 'elbow', 'straight']).isRequired,
  transitionDuration: _propTypes2.default.number.isRequired,
  styles: _propTypes2.default.object
};