'use strict';

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _index = require('../index');

var _index2 = babelHelpers.interopRequireDefault(_index);

// TODO Find a way to meaningfully test `componentWillLeave`
describe('<Link />', function () {
  var linkData = {
    source: {
      x: 123,
      y: 321
    },
    target: {
      x: 456,
      y: 654
    }
  };

  var mockProps = {
    linkData: linkData,
    pathFunc: 'diagonal',
    orientation: 'horizontal',
    transitionDuration: 500,
    styles: {}
  };

  jest.spyOn(_index2.default.prototype, 'drawPath');
  jest.spyOn(_index2.default.prototype, 'diagonalPath');
  jest.spyOn(_index2.default.prototype, 'elbowPath');
  jest.spyOn(_index2.default.prototype, 'straightPath');
  jest.spyOn(_index2.default.prototype, 'applyOpacity');

  // Clear method spies on prototype after each test
  afterEach(function () {
    return jest.clearAllMocks();
  });

  it('applies the base className', function () {
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, mockProps));

    expect(renderedComponent.prop('className')).toBe('linkBase');
  });

  it('applies `props.styles` when defined', function () {
    var initialStyle = { opacity: 0 }; // state.initialStyle
    var fixture = babelHelpers.extends({}, initialStyle, { stroke: '#777', strokeWidth: 2 });
    var renderedComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      styles: fixture
    })));

    expect(renderedComponent.prop('style')).toEqual(fixture);
  });

  it('calls the appropriate path func based on `props.pathFunc`', function () {
    var diagonalComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, mockProps));
    var elbowComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      pathFunc: 'elbow'
    })));
    var straightComponent = (0, _enzyme.shallow)(_react2.default.createElement(_index2.default, babelHelpers.extends({}, mockProps, {
      pathFunc: 'straight'
    })));

    expect(diagonalComponent.instance().diagonalPath).toHaveBeenCalled();
    expect(elbowComponent.instance().elbowPath).toHaveBeenCalled();
    expect(straightComponent.instance().straightPath).toHaveBeenCalled();
    expect(_index2.default.prototype.drawPath).toHaveBeenCalledTimes(3);
  });

  it('returns an appropriate elbowPath according to `props.orientation`', function () {
    expect(_index2.default.prototype.elbowPath(linkData, 'horizontal')).toBe('M' + linkData.source.y + ',' + linkData.source.x + 'V' + linkData.target.x + 'H' + linkData.target.y);
    expect(_index2.default.prototype.elbowPath(linkData, 'vertical')).toBe('M' + linkData.source.x + ',' + linkData.source.y + 'V' + linkData.target.y + 'H' + linkData.target.x);
  });

  it('returns an appropriate straightPath according to `props.orientation`', function () {
    expect(_index2.default.prototype.straightPath(linkData, 'horizontal')).toBe('M' + linkData.source.y + ',' + linkData.source.x + 'L' + linkData.target.y + ',' + linkData.target.x);
    expect(_index2.default.prototype.straightPath(linkData, 'vertical')).toBe('M' + linkData.source.x + ',' + linkData.source.y + 'L' + linkData.target.x + ',' + linkData.target.y);
  });

  it('fades in once it has been mounted', function () {
    var fixture = 1;
    var renderedComponent = (0, _enzyme.mount)(_react2.default.createElement(_index2.default, mockProps));

    expect(renderedComponent.instance().applyOpacity).toHaveBeenCalledWith(fixture);
  });
});