'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = babelHelpers.interopRequireDefault(_propTypes);

var _uuid = require('uuid');

var _uuid2 = babelHelpers.interopRequireDefault(_uuid);

var _d = require('d3');

require('./style.css');

var Node = function (_React$Component) {
  babelHelpers.inherits(Node, _React$Component);

  function Node(props) {
    babelHelpers.classCallCheck(this, Node);

    var _this = babelHelpers.possibleConstructorReturn(this, _React$Component.call(this, props));

    var parent = props.nodeData.parent;

    var originX = parent ? parent.x : 0;
    var originY = parent ? parent.y : 0;

    _this.state = {
      transform: _this.setTransformOrientation(originX, originY),
      initialStyle: {
        opacity: 0
      }
    };

    _this.handleClick = _this.handleClick.bind(_this);
    return _this;
  }

  Node.prototype.componentDidMount = function componentDidMount() {
    var _props$nodeData = this.props.nodeData,
        x = _props$nodeData.x,
        y = _props$nodeData.y;

    var transform = this.setTransformOrientation(x, y);

    this.applyTransform(transform);
  };

  Node.prototype.componentWillUpdate = function componentWillUpdate(nextProps) {
    var transform = this.setTransformOrientation(nextProps.nodeData.x, nextProps.nodeData.y);
    this.applyTransform(transform);
  };

  Node.prototype.setTransformOrientation = function setTransformOrientation(x, y) {
    return this.props.orientation === 'horizontal' ? 'translate(' + y + ',' + x + ')' : 'translate(' + x + ',' + y + ')';
  };

  Node.prototype.applyTransform = function applyTransform(transform) {
    var opacity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
    var done = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {};
    var transitionDuration = this.props.transitionDuration;


    (0, _d.select)(this.node).transition().duration(transitionDuration).attr('transform', transform).style('opacity', opacity).each('end', done);
  };

  Node.prototype.handleClick = function handleClick() {
    this.props.onClick(this.props.nodeData.id);
  };

  Node.prototype.componentWillLeave = function componentWillLeave(done) {
    var parent = this.props.nodeData.parent;

    var originX = parent ? parent.x : 0;
    var originY = parent ? parent.y : 0;
    var transform = this.setTransformOrientation(originX, originY);

    this.applyTransform(transform, 0, done);
  };

  Node.prototype.render = function render() {
    var _this2 = this;

    var _props = this.props,
        nodeData = _props.nodeData,
        styles = _props.styles;

    var nodeStyle = nodeData._children ? babelHelpers.extends({}, styles.node) : babelHelpers.extends({}, styles.leafNode);
    return _react2.default.createElement(
      'g',
      {
        key: this.props.key,
        id: nodeData.id,
        ref: function ref(n) {
          _this2.node = n;
        },
        style: this.state.initialStyle,
        className: nodeData._children ? 'nodeBase' : 'leafNodeBase',
        transform: this.state.transform,
        onClick: this.handleClick
      },
      _react2.default.createElement(
        'text',
        {
          className: 'nodeNameBase',
          textAnchor: this.props.textAnchor,
          style: nodeStyle.name,
          x: '10',
          y: '-10',
          dy: '.35em'
        },
        this.props.name
      ),
      _react2.default.createElement('circle', {
        r: this.props.circleRadius,
        style: nodeStyle.circle
      }),
      _react2.default.createElement(
        'text',
        {
          className: 'nodeAttributesBase',
          y: '0',
          textAnchor: this.props.textAnchor,
          style: nodeStyle.attributes
        },
        this.props.attributes && Object.keys(this.props.attributes).map(function (labelKey) {
          return _react2.default.createElement(
            'tspan',
            { x: '10', dy: '1.2em', key: _uuid2.default.v4() },
            labelKey,
            ': ',
            _this2.props.attributes[labelKey]
          );
        })
      )
    );
  };

  return Node;
}(_react2.default.Component);

exports.default = Node;


Node.defaultProps = {
  textAnchor: 'start',
  attributes: undefined,
  styles: {
    node: {
      circle: {},
      name: {},
      attributes: {}
    },
    leafNode: {
      circle: {},
      name: {},
      attributes: {}
    }
  }
};

Node.propTypes = {
  nodeData: _propTypes2.default.object.isRequired,
  orientation: _propTypes2.default.oneOf(['horizontal', 'vertical']).isRequired,
  transitionDuration: _propTypes2.default.number.isRequired,
  onClick: _propTypes2.default.func.isRequired,
  name: _propTypes2.default.string.isRequired,
  attributes: _propTypes2.default.object,
  textAnchor: _propTypes2.default.string,
  circleRadius: _propTypes2.default.number.isRequired,
  styles: _propTypes2.default.object
};