'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _react = require('react');

var _react2 = babelHelpers.interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = babelHelpers.interopRequireDefault(_propTypes);

var _reactTransitionGroup = require('react-transition-group');

var _d = require('d3');

var _clone = require('clone');

var _clone2 = babelHelpers.interopRequireDefault(_clone);

var _deepEqual = require('deep-equal');

var _deepEqual2 = babelHelpers.interopRequireDefault(_deepEqual);

var _uuid = require('uuid');

var _uuid2 = babelHelpers.interopRequireDefault(_uuid);

var _Node = require('../Node');

var _Node2 = babelHelpers.interopRequireDefault(_Node);

var _Link = require('../Link');

var _Link2 = babelHelpers.interopRequireDefault(_Link);

require('./style.css');

var Tree = function (_React$Component) {
  babelHelpers.inherits(Tree, _React$Component);

  function Tree(props) {
    babelHelpers.classCallCheck(this, Tree);

    var _this = babelHelpers.possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.state = {
      initialRender: true,
      data: _this.assignInternalProperties((0, _clone2.default)(props.data))
    };
    _this.findNodesById = _this.findNodesById.bind(_this);
    _this.collapseNode = _this.collapseNode.bind(_this);
    _this.handleNodeToggle = _this.handleNodeToggle.bind(_this);
    _this.handleOnClickCb = _this.handleOnClickCb.bind(_this);
    return _this;
  }

  Tree.prototype.componentDidMount = function componentDidMount() {
    this.bindZoomListener(this.props);
    // TODO find better way of setting initialDepth, re-render here is suboptimal
    this.setState({ initialRender: false }); // eslint-disable-line
  };

  Tree.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    // Clone new data & assign internal properties
    if (!(0, _deepEqual2.default)(this.props.data, nextProps.data)) {
      this.setState({
        data: this.assignInternalProperties((0, _clone2.default)(nextProps.data))
      });
    }

    // If zoom-specific props change -> rebind listener with new values
    if (!(0, _deepEqual2.default)(this.props.translate, nextProps.translate) || !(0, _deepEqual2.default)(this.props.scaleExtent, nextProps.scaleExtent)) {
      this.bindZoomListener(nextProps);
    }
  };

  /**
   * setInitialTreeDepth - Description
   *
   * @param {array} nodeSet Array of nodes generated by `generateTree`
   * @param {number} initialDepth Maximum initial depth the tree should render
   *
   * @return {void}
   */


  Tree.prototype.setInitialTreeDepth = function setInitialTreeDepth(nodeSet, initialDepth) {
    nodeSet.forEach(function (n) {
      n._collapsed = n.depth >= initialDepth;
    });
  };

  /**
   * bindZoomListener - If `props.zoomable`, binds a listener for
   * "zoom" events to the SVG and sets scaleExtent to min/max
   * specified in `props.scaleExtent`.
   *
   * @return {void}
   */


  Tree.prototype.bindZoomListener = function bindZoomListener(props) {
    var zoomable = props.zoomable,
        scaleExtent = props.scaleExtent,
        translate = props.translate;

    var svg = (0, _d.select)('.rd3t-svg');
    var g = (0, _d.select)('.rd3t-g');

    if (zoomable) {
      svg.call(_d.behavior.zoom().scaleExtent([scaleExtent.min, scaleExtent.max]).on('zoom', function () {
        g.attr('transform', 'translate(' + _d.event.translate + ') scale(' + _d.event.scale + ')');
      })
      // Offset so that first pan and zoom does not jump back to [0,0] coords
      .translate([translate.x, translate.y]));
    }
  };

  /**
   * assignInternalProperties - Assigns internal properties to each node in the
   * `data` set that are required for tree manipulation and returns
   * a new `data` array.
   *
   * @param {array} data Hierarchical tree data
   *
   * @return {array} `data` array with internal properties added
   */


  Tree.prototype.assignInternalProperties = function assignInternalProperties(data) {
    var _this2 = this;

    return data.map(function (node) {
      node.id = _uuid2.default.v4();
      node._collapsed = false;
      // if there are children, recursively assign properties to them too
      if (node.children && node.children.length > 0) {
        node.children = _this2.assignInternalProperties(node.children);
        node._children = node.children;
      }
      return node;
    });
  };

  /**
   * findNodesById - Description
   *
   * @param {string} nodeId The `node.id` being searched for
   * @param {array} nodeSet Array of `node` objects
   * @param {array} hits Accumulator for matches, passed between recursive calls
   *
   * @return {array} Set of nodes matching `nodeId`
   */
  // TODO Refactor this into a more readable/reasonable recursive depth-first walk.


  Tree.prototype.findNodesById = function findNodesById(nodeId, nodeSet, hits) {
    var _this3 = this;

    if (hits.length > 0) {
      return hits;
    }

    hits = hits.concat(nodeSet.filter(function (node) {
      return node.id === nodeId;
    }));

    nodeSet.forEach(function (node) {
      if (node._children && node._children.length > 0) {
        hits = _this3.findNodesById(nodeId, node._children, hits);
        return hits;
      }
      return hits;
    });

    return hits;
  };

  /**
   * collapseNode - Recursively sets the `_collapsed` property of
   * the passed `node` object and its children to `true`.
   *
   * @param {object} node Node object with custom properties
   *
   * @return {void}
   */


  Tree.prototype.collapseNode = function collapseNode(node) {
    var _this4 = this;

    node._collapsed = true;
    if (node._children && node._children.length > 0) {
      node._children.forEach(function (child) {
        _this4.collapseNode(child);
      });
    }
  };

  /**
   * expandNode - Sets the `_collapsed` property of
   * the passed `node` object to `false`.
   *
   * @param {type} node Node object with custom properties
   *
   * @return {void}
   */


  Tree.prototype.expandNode = function expandNode(node) {
    node._collapsed = false;
  };

  /**
   * handleNodeToggle - Finds the node matching `nodeId` and
   * expands/collapses it, depending on the current state of
   * its `_collapsed` property.
   * `setState` callback receives targetNode and handles
   * `props.onClick` if defined.
   *
   * @param {string} nodeId A node object's `id` field.
   *
   * @return {void}
   */


  Tree.prototype.handleNodeToggle = function handleNodeToggle(nodeId) {
    var _this5 = this;

    var data = (0, _clone2.default)(this.state.data);
    var matches = this.findNodesById(nodeId, data, []);
    var targetNode = matches[0];

    if (this.props.collapsible) {
      targetNode._collapsed ? this.expandNode(targetNode) : this.collapseNode(targetNode);
      this.setState({ data: data }, function () {
        return _this5.handleOnClickCb(targetNode);
      });
    } else {
      this.handleOnClickCb(targetNode);
    }
  };

  /**
   * handleOnClickCb - Handles the user-defined `onClick` function
   *
   * @param {object} targetNode Description
   *
   * @return {void}
   */


  Tree.prototype.handleOnClickCb = function handleOnClickCb(targetNode) {
    var onClick = this.props.onClick;

    if (onClick && typeof onClick === 'function') {
      onClick((0, _clone2.default)(targetNode));
    }
  };

  /**
   * generateTree - Generates tree elements (`nodes` and `links`) by
   * grabbing the rootNode from `this.state.data[0]`.
   * Restricts tree depth to `props.initialDepth` if defined and if this is
   * the initial render of the tree.
   *
   * @return {object} Object containing `nodes` and `links`.
   */


  Tree.prototype.generateTree = function generateTree() {
    var _props = this.props,
        initialDepth = _props.initialDepth,
        depthFactor = _props.depthFactor,
        separation = _props.separation,
        nodeSize = _props.nodeSize,
        orientation = _props.orientation;


    var tree = _d.layout.tree().nodeSize(orientation === 'horizontal' ? [nodeSize.y, nodeSize.x] : [nodeSize.x, nodeSize.y]).separation(function (a, b) {
      return (0, _deepEqual2.default)(a.parent, b.parent) ? separation.siblings : separation.nonSiblings;
    }).children(function (d) {
      return d._collapsed ? null : d._children;
    });

    var rootNode = this.state.data[0];
    var nodes = tree.nodes(rootNode);
    var links = tree.links(nodes);

    // set `initialDepth` on first render if specified
    if (initialDepth !== undefined && this.state.initialRender) {
      this.setInitialTreeDepth(nodes, initialDepth);
    }

    if (depthFactor) {
      nodes.forEach(function (node) {
        node.y = node.depth * depthFactor;
      });
    }

    return { nodes: nodes, links: links };
  };

  Tree.prototype.render = function render() {
    var _this6 = this;

    var _generateTree = this.generateTree(),
        nodes = _generateTree.nodes,
        links = _generateTree.links;

    var _props2 = this.props,
        orientation = _props2.orientation,
        translate = _props2.translate,
        pathFunc = _props2.pathFunc,
        transitionDuration = _props2.transitionDuration,
        zoomable = _props2.zoomable,
        circleRadius = _props2.circleRadius,
        styles = _props2.styles;


    return _react2.default.createElement(
      'div',
      { className: 'rd3t-tree-container ' + (zoomable ? 'rd3t-grabbable' : undefined) },
      _react2.default.createElement(
        'svg',
        { className: 'rd3t-svg', width: '100%', height: '100%' },
        _react2.default.createElement(
          _reactTransitionGroup.TransitionGroup,
          {
            component: 'g',
            className: 'rd3t-g',
            transform: 'translate(' + translate.x + ',' + translate.y + ')'
          },
          links.map(function (linkData) {
            return _react2.default.createElement(_Link2.default, {
              key: _uuid2.default.v4(),
              orientation: orientation,
              pathFunc: pathFunc,
              linkData: linkData,
              transitionDuration: transitionDuration,
              styles: styles.links
            });
          }),
          nodes.map(function (nodeData) {
            return _react2.default.createElement(_Node2.default, {
              key: nodeData.id,
              orientation: orientation,
              transitionDuration: transitionDuration,
              textAnchor: 'start',
              nodeData: nodeData,
              name: nodeData.name,
              attributes: nodeData.attributes,
              onClick: _this6.handleNodeToggle,
              circleRadius: circleRadius,
              styles: styles.nodes
            });
          })
        )
      )
    );
  };

  return Tree;
}(_react2.default.Component);

exports.default = Tree;


Tree.defaultProps = {
  onClick: undefined,
  orientation: 'horizontal',
  translate: { x: 0, y: 0 },
  pathFunc: 'diagonal',
  transitionDuration: 500,
  depthFactor: undefined,
  collapsible: true,
  initialDepth: undefined,
  zoomable: true,
  scaleExtent: { min: 0.1, max: 1 },
  nodeSize: { x: 140, y: 140 },
  separation: { siblings: 1, nonSiblings: 2 },
  circleRadius: 10,
  styles: {}
};

Tree.propTypes = {
  data: _propTypes2.default.array.isRequired,
  onClick: _propTypes2.default.func,
  orientation: _propTypes2.default.oneOf(['horizontal', 'vertical']),
  translate: _propTypes2.default.shape({
    x: _propTypes2.default.number,
    y: _propTypes2.default.number
  }),
  pathFunc: _propTypes2.default.oneOf(['diagonal', 'elbow', 'straight']),
  transitionDuration: _propTypes2.default.number,
  depthFactor: _propTypes2.default.number,
  collapsible: _propTypes2.default.bool,
  initialDepth: _propTypes2.default.number,
  zoomable: _propTypes2.default.bool,
  scaleExtent: _propTypes2.default.shape({
    min: _propTypes2.default.number,
    max: _propTypes2.default.number
  }),
  nodeSize: _propTypes2.default.shape({
    x: _propTypes2.default.number,
    y: _propTypes2.default.number
  }),
  separation: _propTypes2.default.shape({
    siblings: _propTypes2.default.number,
    nonSiblings: _propTypes2.default.number
  }),
  circleRadius: _propTypes2.default.number,
  styles: _propTypes2.default.shape({
    nodes: _propTypes2.default.object,
    links: _propTypes2.default.object
  })
};